import { debounce } from 'lodash';
import { useMemo, useState, useCallback } from 'react';
import { t } from '../../utils/i18n.mjs';
import { itemToString, fuzzyFind } from './filter.mjs';
import { useLatestAsyncCall, StaleResultError } from './useLatestAsyncCall.mjs';

const asyncNoop = () => Promise.resolve([]);
function useOptions(rawOptions, createCustomValue) {
  const isAsync = typeof rawOptions === "function";
  const loadOptions = useLatestAsyncCall(isAsync ? rawOptions : asyncNoop);
  const debouncedLoadOptions = useMemo(
    () => debounce((searchTerm) => {
      return loadOptions(searchTerm).then((options) => {
        setAsyncOptions(options);
        setAsyncLoading(false);
        setAsyncError(false);
      }).catch((error) => {
        if (!(error instanceof StaleResultError)) {
          setAsyncError(true);
          setAsyncLoading(false);
          if (error) {
            console.error("Error loading async options for Combobox", error);
          }
        }
      });
    }, 200),
    [loadOptions]
  );
  const [asyncOptions, setAsyncOptions] = useState([]);
  const [asyncLoading, setAsyncLoading] = useState(false);
  const [asyncError, setAsyncError] = useState(false);
  const [userTypedSearch, setUserTypedSearch] = useState("");
  const addCustomValue = useCallback(
    (opts) => {
      let currentOptions = opts;
      if (createCustomValue && userTypedSearch) {
        const customValueExists = opts.some((opt) => opt.value === userTypedSearch);
        if (!customValueExists) {
          currentOptions.unshift({
            label: userTypedSearch,
            value: userTypedSearch,
            description: t("combobox.custom-value.description", "Use custom value")
          });
        }
      }
      return currentOptions;
    },
    [createCustomValue, userTypedSearch]
  );
  const updateOptions = useCallback(
    (inputValue) => {
      setUserTypedSearch(inputValue);
      if (isAsync) {
        setAsyncLoading(true);
        debouncedLoadOptions(inputValue);
      }
    },
    [debouncedLoadOptions, isAsync]
  );
  const stringifiedOptions = useMemo(() => {
    return isAsync ? [] : rawOptions.map(itemToString);
  }, [isAsync, rawOptions]);
  const filteredOptions = useMemo(() => {
    if (isAsync) {
      return asyncOptions;
    }
    return fuzzyFind(rawOptions, stringifiedOptions, userTypedSearch);
  }, [asyncOptions, isAsync, rawOptions, stringifiedOptions, userTypedSearch]);
  const [finalOptions, groupStartIndices] = useMemo(() => {
    const { options, groupStartIndices: groupStartIndices2 } = sortByGroup(filteredOptions);
    return [addCustomValue(options), groupStartIndices2];
  }, [filteredOptions, addCustomValue]);
  return { options: finalOptions, groupStartIndices, updateOptions, asyncLoading, asyncError };
}
function sortByGroup(options) {
  var _a;
  const groupedOptions = /* @__PURE__ */ new Map();
  for (const option of options) {
    const groupExists = groupedOptions.has(option.group);
    if (groupExists) {
      (_a = groupedOptions.get(option.group)) == null ? undefined : _a.push(option);
    } else {
      groupedOptions.set(option.group, [option]);
    }
  }
  const groupStartIndices = /* @__PURE__ */ new Map();
  let currentIndex = 0;
  let reorganizeOptions = [];
  for (const [group, groupOptions] of groupedOptions) {
    if (!group) {
      continue;
    }
    groupStartIndices.set(group, currentIndex);
    reorganizeOptions = reorganizeOptions.concat(groupOptions);
    currentIndex += groupOptions.length;
  }
  const undefinedGroupOptions = groupedOptions.get(undefined);
  if (undefinedGroupOptions) {
    groupStartIndices.set("undefined", currentIndex);
    reorganizeOptions = reorganizeOptions.concat(undefinedGroupOptions);
  }
  return {
    options: reorganizeOptions,
    groupStartIndices
  };
}

export { useOptions };
//# sourceMappingURL=useOptions.mjs.map
